#!/bin/bash
#
# Common functions for bochs-tools.
# 
# For a brief summary, run bxmdisk.sh -h
#
# $Author: bablokb $
# $Revision: 1.33 $
#
# License: GPL2
# -----------------------------------------------------------------------------

version=1.6.4

# version information   --------------------------------------------------------

getVersion() {
  echo "$version"
}

# show version   ---------------------------------------------------------------

showVersion() {
  echo -en "\n$1: $2, version `getVersion`\n\n"
}

# copyright message   ---------------------------------------------------------

showCopyright() {
  showVersion $1 "$2"
  echo -en "\n Copyright (C) 2003 Bernhard Bablok, mail@bablokb.de\n" \
           "This is free software; see the source for copying conditions.\n" \
           "There is NO warranty; not even for MERCHANTABILITY or\n" \
           "FITNESS FOR A PARTICULAR PURPOSE.\n"
 exit 3
}

# parse geometry variables   -------------------------------------------------

parseGeometry() {  # anything simpler??
  cyl=$1
  heads=$2
  spt=$3
}

# set geometry variables   ----------------------------------------------------

setGeometry() {
  local file=$1
  if [ -n "$file" ]; then
     parseGeometry `bxtptinfo -g $file`
  fi
  [ -z "$heads" ] && let heads=16
  [ -z "$spt" ]   && let spt=63
  let sectPerCyl=heads*spt
  let bytesPerCyl=512*sectPerCyl
  let blocksPerCyl=sectPerCyl/2
}

# execute a command   ----------------------------------------------------------

do_cmd() {
  local cmd=$1
  if [ $quiet -eq 0 ]; then
    echo "executing: $cmd" >&2
  fi
  if [ $simulate -eq 0 ]; then
    eval $cmd
    return $?
  fi
  return 0
}

# return size in blocks (argument: size in bytes, KB, MB, GB)   ---------------

getSize() {
  local size=$1
  case $size in
    *[bB]) let size=${size%[bb]}/2;;
    *[kK]) let size=${size%[kK]};;
    *[kK][dD]) let size=(1000*${size%[kK][dD]}-1)/1024+1;;
    *[mM]) let size=1024*${size%[mM]};;
    *[mM][dD]) let size=977*${size%[mM][dD]};;
    *[gG]) let size=1048576*${size%[gG]};;
    *[gG][dD]) let size=1000000*${size%[gG][dD]};;
    *) [ $size -gt 0 ] && let size=(size-1)/1024+1;;
  esac
  echo $size
}

# check for sfdisk   ----------------------------------------------------------

checkSfdisk() {
  if ! type -p sfdisk > /dev/null; then
    echo "error: sfdisk not found. You need sfdisk to run this program." >&2
    echo "sfdisk is part of util-linux and usually installed on every system." >&2
    exit 1
  fi
}

# create temporary mount-point   ----------------------------------------------
# 
# first argument: variable for mount-point

createMountPoint() {
  if ! do_cmd "$1=\`mktemp -d -q /tmp/bxt.XXXXXX\`"; then
    echo "error: could not create temporary mount-point!" >&2
    return 1
  fi
  return 0
}

# mount partition using bxtmount   --------------------------------------------
# 
# first argument: partition
# second argument: mount-point

doMount() {
  local cmd="bxtmount -q"
  [ $simulate -eq 1 ] && cmd="$cmd -s"
  [ -n "$extraOptions" ] && cmd="$cmd -x $extraOptions"
  if ! do_cmd "$cmd $1 $2"; then
   return 1                   # error messages from bxtmount
  fi
  return 0
}

# umount and remove temporary mount-point   -----------------------------------
# 
# first argument: mount-point

doUmount() {
  mount | grep -qs $1 && do_cmd "umount $1"
  if mount | grep -qs $1; then
    echo "error: umount failed!" >&2
  else
    rm -fr $1
  fi
}

# query free loop-device   ----------------------------------------------------

getLoopDevice() {
  for i in 0 1 2 3 4 5 6 7; do
    if ! losetup /dev/loop$i 1>/dev/null 2>&1; then
      echo "/dev/loop$i"
      return 0;
    fi
  done
  return 1
}

# create temporary file   -----------------------------------------------------
#
# first argument: variable-name of temporary file

createTempFile() {
  if ! do_cmd "$1=\`mktemp -q /tmp/bxt.XXXXXX\`"; then
    echo "error: could not create temporary file!" >&2
    return 1
  fi
  return 0
}

# losetup of bxt-partition   --------------------------------------------------
#
# usage: setupLoopDevice 2 hda.img
#        setupLoopDevice hda.img:2
#
# On success, bxtlosetup will echo the name of the loop-device

setupLoopDevice() {
  local partition=$1, imageName=$2
  if [ -z "$2" ]; then
    imageName=${1%:*}
    partition=${1#*:}
  fi    
  local offset=`bxtptinfo -p $partition $imageName`
  if [ -z "$offset" ]; then
    return 1
  fi
  offset=${offset%% *}
  if [ $offset -eq 0 ]; then
    return 1
  fi
  if ! loopDevice=`getLoopDevice`; then
    return 1
  fi
  if do_cmd "losetup -o $offset $loopDevice $imageName" 1>/dev/null 2>&1; then
    echo $loopDevice
    return 0
  else
    return $?
  fi
}